#!/bin/bash

. /etc/default/${EPAGES_PROJECT:-epages6}
. /var/epages/functions/fun-pkg.sh
export pkg_installed required_pkg_version mktempdir

PERL_EXE=$EPAGES_PERL/bin/perl

ERROR=$(ls Makefile.PL patch.pl $PERL_EXE 2>&1 >/dev/null)
RESULT=$?
if [[ $RESULT != 0 ]] ; then
  echo "patch.sh: $ERROR"
  exit $RESULT
fi

hostname=$(hostname)
PATCH_TAG=$(awk -F= '/^PATCH_TAG[[:space:]]*=/{gsub(/[[:space:]]+/,""); print $2}' Makefile.PL)
LOG_DIR=${EPAGES_PATCHES:-"$EPAGES_SHARED/Patches"}/var-$PATCH_TAG/logs/$hostname
mkdir -m 755 -p $LOG_DIR || exit 1

#------------------------------------------------------------------------------
# test if package is installed in required version
#------------------------------------------------------------------------------

test_package() {
  local PACKAGE=$1
  local REQ_VERSION=$2

  local VER
  if [[ -f /etc/debian_version ]] && dpkg --status $PACKAGE >/dev/null 2>&1 ; then
    VER=$(dpkg-query --show -f '${Version}\n' $PACKAGE | sed 's,[~-].*,,' | sort -V | tail -1)
  elif [[ -f /etc/redhat-release ]] && rpm --quiet -q $PACKAGE ; then
    VER=$(rpm -q $PACKAGE --qf '[%{VERSION}\n]' | sort -V | tail -1)
  fi

  [[ -n $VER ]] || return 1
  [[ $(echo -e "$VER\n$REQ_VERSION" | sort -V | head -1) = $REQ_VERSION ]]
  echo $VER
  return $?
}

#------------------------------------------------------------------------------
# check if epages-release is up-to-date
#------------------------------------------------------------------------------

# test if function required_pkg_version exists
if ! type -t required_pkg_version >/dev/null ; then
  INSTALLED_VERSION=$(test_package epages-release 6.17.36)
  echo "Package epages-release (${INSTALLED_VERSION:-not found}) is outdated, exiting."
  echo "First update epages-release by executing:"
  if [[ -f /etc/debian_version && -z $INSTALLED_VERSION ]] ; then
    echo "  curl -O  http://epages.com/repo/epages/apt/epages-release_all.deb"
    echo "  dpkg -i epages-release_all.deb ; apt-get -y update"
  else
    echo "  rpm -hUv http://epages.com/repo/epages/yum/epages-release.noarch.rpm"
  fi
  echo "Then re-run: $0 $@"
  exit 1
fi

#------------------------------------------------------------------------------
# check if epages-perl is up-to-date
#------------------------------------------------------------------------------

# do not update perl if perl flex server is running
service ePagesConfigServer status >/dev/null 2>&1 && SKIP_PERL_UPDATE=exit

# if patch.sh was executed by a packaging tool we can't run packaging tools
started_by_rpm $$ && SKIP_PERL_UPDATE=rpm

if ! required_pkg_version epages-perl 6.17.36 ; then
  if [[ $SKIP_PERL_UPDATE = rpm ]] ; then
    [[ $EPAGES_DO_PATCH = 0 ]] && SKIP_PERL_UPDATE=skip_patch || SKIP_PERL_UPDATE=exit
  fi
  PKG_VERSION=$(pkg_installed -V epages-perl 2>/dev/null | tail -1)
  if [[ $SKIP_PERL_UPDATE = skip_patch ]] ; then
    # do nothing if we just unpack the package without starting the patch
    true
  elif [[ -n $SKIP_PERL_UPDATE ]] ; then
    # either show update message
    PKG_VERSION=${PKG_VERSION:+-$PKG_VERSION}
    echo "Package epages-perl${PKG_VERSION:+-$PKG_VERSION} is outdated."
    echo "First update epages-perl by executing:"
    echo "  ENABLE_REPO=epages-setup /var/epages/install-RPM.sh epages-perl"
    echo "Then re-run: $0 $@"
    [[ $SKIP_PERL_UPDATE = exit ]] && exit 1
  else
    # or update epages-perl
    echo "ENABLE_REPO=epages-setup /var/epages/install-RPM.sh epages-perl"
    ENABLE_REPO=epages-setup /var/epages/install-RPM.sh epages-perl
  fi
fi

#------------------------------------------------------------------------------
# run perl
#------------------------------------------------------------------------------

date=$(date +'%Y-%h-%d-%H.%M.%S')

TEMP_FILE=$(mktempdir epages-$PATCH_TAG)/patch.pl.success
rm -f ${TEMP_FILE%/*}/patch.pl.*

$EPAGES_PERL/bin/perl patch.pl "$@" 2>&1 | tee $LOG_DIR/$date-${1////-}.log

# return success or failure

RESULT=${PIPESTATUS[0]}
[[ -f $(mktempdir epages-$PATCH_TAG)/patch.pl.success ]] && RESULT=0
[[ -f $(mktempdir epages-$PATCH_TAG)/patch.pl.failure ]] && RESULT=1

exit $RESULT
